(**************************************************************************

   	This file is part of the AtomicDensityMatrix package for Mathematica
    Copyright (C) 2004-2014 Simon M. Rochester <simon@rochesterscientific.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************)

(* This package provides a database for information about atomic levels.
Adapted from the CityData package by John M. Novak
*)

Quiet[
    BeginPackage["AtomicDensityMatrix`AtomicData`", {
        "PhysicalConstants`",
        "Units`",
        "AtomicDensityMatrix`",
        "AtomicDensityMatrix`Common`"}],
    General::obspkg
];

(*
NuclearSpin::usage = ""
J::usage = ""
L::usage = ""
S::usage = ""
NaturalWidth::usage = ""
Energy::usage = ""
HyperfineA::usage = ""
HyperfineB::usage = ""
Parity::usage = ""
Even::usage = ""
Odd::usage = "" 

ElementNames::usage = ""
ElementPosition::usage = ""
GetElementSymbol::usage = ""
GetAtomicNumber::usage = ""
GetElementName::usage = ""
*)

Begin["`Private`"];

$AtomicDataFields = {ElementName, ElementSymbol, AtomicNumber, AtomicWeight, Abundance, NuclearSpin, J, L, S, NaturalWidth, Energy, HyperfineA, HyperfineB, Parity, Wavenumber};

AtomicData::alternates =
    "Requested state not in database.  Possible alternatives: `1`";

Options[AtomicData] = {StateLabel -> None};

AtomicData[symbol_Symbol] :=
    Select[Map[#[[1, 1, 1]]&, DownValues[symbol]], Head[#] === List &];

AtomicData[] := Union[Flatten[AtomicData /@ $AtomicDataFields, 1]];

AtomicData[stat_List] /; Length[stat] < 4 :=
    Select[AtomicData[], Take[#, Length[stat]] === stat&];

AtomicData[stat_List, opts___Rule] :=
    Module[{dat},
        dat = Map[AtomicData[stat, #, opts]&, $AtomicDataFields];
        dat = Select[dat, Not[MemberQ[$AtomicDataFields, Head[Last[#]]]]&]
    ];

AtomicData[stat_List, symbol_Symbol, opts___Rule] := Module[{lab},
    lab = StateLabel /. {opts} /. Options[AtomicData];
    If[lab === None,
        symbol -> symbol[stat],
        symbol[lab] -> symbol[stat]
    ]
];

AtomicData[stat_List, list_List, opts___Rule] := AtomicData[stat, #, opts]& /@ list;

checkclose[stat_List, list_List] :=
    Select[list, (Head[#] === List&&First[#] === cit)&];

checkclose[cit_List, list_List] :=
    Select[list, (Head[#] === String&&# === First[cit])&];

(*

Options[CityDistance] =
	{CityDistanceMethod->SphericalDistance}

CityDistance[city1_,city2_,opts___] :=
	Module[{meth,pos},
		{meth} = {CityDistanceMethod}/.{opts}/.Options[CityDistance];
		pos = {CityData[city1,CityPosition],
			CityData[city2,CityPosition]};
		N[Apply[meth,pos]]/;VectorQ[Flatten[pos]]
	]
*)

(* Atomic Information *)

ElementNames = {{"H", 1, "Hydrogen"}, {"He", 2, "Helium"}, {"Li", 3, "Lithium"}, {"Be", 4,
    "Beryllium"}, {"B", 5, "Boron"}, {"C", 6, "Carbon"}, {"N", 7,
    "Nitrogen"}, {"O", 8, "Oxygen"}, {"F", 9, "Fluorine"}, {"Ne", 10,
    "Neon"}, {"Na", 11, "Sodium"}, {"Mg", 12, "Magnesium"}, {"Al", 13,
    "Aluminium"}, {"Si", 14, "Silicon"}, {"P", 15, "Phosphorus"}, {"S", 16,
    "Sulfur"}, {"Cl", 17, "Chlorine"}, {"Ar", 18, "Argon"}, {"K", 19,
    "Potassium"}, {"Ca", 20, "Calcium"}, {"Sc", 21, "Scandium"}, {"Ti", 22,
    "Titanium"}, {"V", 23, "Vanadium"}, {"Cr", 24, "Chromium"}, {"Mn", 25,
    "Manganese"}, {"Fe", 26, "Iron"}, {"Co", 27, "Cobalt"}, {"Ni", 28,
    "Nickel"}, {"Cu", 29, "Copper"}, {"Zn", 30, "Zinc"}, {"Ga", 31,
    "Gallium"}, {"Ge", 32, "Germanium"}, {"As", 33, "Arsenic"}, {"Se", 34,
    "Selenium"}, {"Br", 35, "Bromine"}, {"Kr", 36, "Krypton"}, {"Rb", 37,
    "Rubidium"}, {"Sr", 38, "Strontium"}, {"Y", 39, "Yttrium"}, {"Zr", 40,
    "Zirconium"}, {"Nb", 41, "Niobium"}, {"Mo", 42, "Molybdenum"}, {"Tc", 43,
    "Technetium"}, {"Ru", 44, "Ruthenium"}, {"Rh", 45, "Rhodium"}, {"Pd", 46,
    "Palladium"}, {"Ag", 47, "Silver"}, {"Cd", 48, "Cadmium"}, {"In", 49,
    "Indium"}, {"Sn", 50, "Tin"}, {"Sb", 51, "Antimony"}, {"Te", 52,
    "Tellurium"}, {"I", 53, "Iodine"}, {"Xe", 54, "Xenon"}, {"Cs", 55,
    "Caesium"}, {"Ba", 56, "Barium"}, {"La", 57, "Lanthanum"}, {"Ce", 58,
    "Cerium"}, {"Pr", 59, "Praseodymium"}, {"Nd", 60, "Neodymium"}, {"Pm", 61,
    "Promethium"}, {"Sm", 62, "Samarium"}, {"Eu", 63, "Europium"}, {"Gd", 64,
    "Gadolinium"}, {"Tb", 65, "Terbium"}, {"Dy", 66, "Dysprosium"}, {"Ho",
    67, "Holmium"}, {"Er", 68, "Erbium"}, {"Tm", 69, "Thulium"}, {"Yb", 70,
    "Ytterbium"}, {"Lu", 71, "Lutetium"}, {"Hf", 72, "Hafnium"}, {"Ta", 73,
    "Tantalum"}, {"W", 74, "Tungsten"}, {"Re", 75, "Rhenium"}, {"Os", 76,
    "Osmium"}, {"Ir", 77, "Iridium"}, {"Pt", 78, "Platinum"}, {"Au", 79,
    "Gold"}, {"Hg", 80, "Mercury"}, {"Tl", 81, "Thallium"}, {"Pb", 82,
    "Lead"}, {"Bi", 83, "Bismuth"}, {"Po", 84, "Polonium"}, {"At", 85,
    "Astatine"}, {"Rn", 86, "Radon"}, {"Fr", 87, "Francium"}, {"Ra", 88,
    "Radium"}, {"Ac", 89, "Actinium"}, {"Th", 90, "Thorium"}, {"Pa", 91,
    "Protactinium"}, {"U", 92, "Uranium"}, {"Np", 93, "Neptunium"}, {"Pu", 94,
    "Plutonium"}, {"Am", 95, "Americium"}, {"Cm", 96, "Curium"}, {"Bk", 97,
    "Berkelium"}, {"Cf", 98, "Californium"}, {"Es", 99, "Einsteinium"}, {"Fm",
    100, "Fermium"}, {"Md", 101, "Mendelevium"}, {"No", 102,
    "Nobelium"}, {"Lr", 103, "Lawrencium"}, {"Rf", 104,
    "Rutherfordium"}, {"Db", 105, "Dubnium"}, {"Sg", 106,
    "Seaborgium"}, {"Bh", 107, "Bohrium"}, {"Hs", 108, "Hassium"}, {"Mt", 109,
    "Meitnerium"}};

SpecList = {"S", "P", "D", "F", "G", "H", "I", "K", "L", "M", "N", "O", "Q", "R", "T", "U"};

ElementPosition[label_] := Position[ElementNames, label][[1, 1]];
GetElementSymbol[label_] := ElementNames[[ElementPosition[label], 1]];
GetAtomicNumber[label_] := ElementNames[[ElementPosition[label], 2]];
GetElementName[label_] := ElementNames[[ElementPosition[label], 3]];
GetJ[term_] := Last[term];
GetS[term_] := (First[term]-1)/2;
GetL[term_] := Position[SpecList, term[[2]]][[1, 1]]-1;
GetStateData[state_] := (
    ElementName[state] = GetElementName[state[[1]]];
    ElementSymbol[state] = GetElementSymbol[state[[1]]];
    AtomicNumber[state] = GetAtomicNumber[state[[1]]];
    AtomicWeight[state] = state[[2]];
    J[state] = GetJ[state[[4]]];
    L[state] = GetL[state[[4]]];
    S[state] = GetS[state[[4]]];);


DopplerWidth[gstate_, estate_, temp_] :=
    2 Pi Convert[Sqrt[(Wavenumber[estate]-Wavenumber[gstate])^2(2 BoltzmannConstant temp Kelvin)/(AtomicWeight[gstate] AtomicMassUnit)], Mega 1/Second];

MostProbableSpeed[state_, temp_] :=
    Convert[Sqrt[(2 BoltzmannConstant temp Kelvin)/(AtomicWeight[state] AtomicMassUnit)] /. AtomicData[estate], Centimeter/Second];

(*
	Na, Rb, and Cs data from http://steck.us/alkalidata/ 
	Rb natural abundance from http://www.tracesciences.com/rb.htm
 *)

(* H 

Wavenumbers from http://www.nist.gov/physlab/data/handbook/index.cfm

*)

state = {"H", 1, {{1, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 1/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 2\[Pi] 1420. Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"H", 1, {{2, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 1/2;
NaturalWidth[state] = (1.60 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 82258.9191 1/Centimeter;
HyperfineA[state] = 0 2\[Pi] Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"H", 1, {{2, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 1/2;
NaturalWidth[state] = (1.60 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 82259.2850 1/Centimeter;
HyperfineA[state] = 0 2\[Pi] Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

(* He *)

state = {"He", 2, {{1, "s", 2}}, {1, "S", 0}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 0;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 0;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"He", 2, {{1, "s"}, {2, "s"}}, {3, "S", 1}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 0;
Wavenumber[state] = 159855.9743297 1/Centimeter;
HyperfineA[state] = 0;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"He", 2, {{1, "s"}, {2, "s"}}, {1, "S", 0}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 0;
Wavenumber[state] = 166277.440141 1/Centimeter;
HyperfineA[state] = 0;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"He", 2, {{1, "s"}, {2, "p"}}, {3, "P", 2}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 0;
NaturalWidth[state] = 2 Pi 1.62 Mega 1/Second;  (* Laser Cooling and Trapping by Metcalf and van der Straten *)
Wavenumber[state] = 169086.7664725 1/Centimeter;
HyperfineA[state] = 0;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"He", 2, {{1, "s"}, {2, "p"}}, {3, "P", 1}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 0;
NaturalWidth[state] = 2 Pi 1.62 Mega 1/Second; (* Laser Cooling and Trapping by Metcalf and van der Straten *)
Wavenumber[state] = 169086.8428979 1/Centimeter;
HyperfineA[state] = 0;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"He", 2, {{1, "s"}, {2, "p"}}, {3, "P", 0}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 0;
NaturalWidth[state] = 2 Pi 1.62 Mega 1/Second; (* Laser Cooling and Trapping by Metcalf and van der Straten *)
Wavenumber[state] = 169087.8308131 1/Centimeter;
HyperfineA[state] = 0;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

(* Li *)

state = {"Li", 3, {{1, "s", 2}, {2, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = 92.41;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 401.7520433 (2 Pi) Mega 1/Second; (* Arimondo 1977 RMP*)
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"Li", 3, {{1, "s", 2}, {2, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = 92.41;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (27.2 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 14903.66 1/Centimeter;
HyperfineA[state] = 46.024 (2 Pi) Mega 1/Second; (* Das and Natarajan J.Phys.B 40 (2008) 035001 *)
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"Li", 3, {{1, "s", 2}, {2, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = 92.41;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (27.2 10^-3)^-1 Mega 1/Second ;;
    Wavenumber[state] = 14904.00 1/Centimeter;
HyperfineA[state] = -3.055 (2 Pi) Mega 1/Second; (* Arimondo 1977 RMP*)
HyperfineB[state] = -0.221 (2 Pi) Mega 1/Second; (* Arimondo 1977 RMP*)
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;


(* Na *)

state = {"Na", 23, {"Ne", {3, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 2\[Pi] 885.81306440 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"Na", 23, {"Ne", {3, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (16.299 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 16956.170250 1/Centimeter;
HyperfineA[state] = 2\[Pi] 94.44 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"Na", 23, {"Ne", {3, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (16.2492 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 16973.366160 1/Centimeter;
HyperfineA[state] = 2\[Pi] 18.534 Mega 1/Second;
HyperfineB[state] = 2\[Pi] 2.724 Mega 1/Second;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

(* K 
Natural abundance from http://physics.nist.gov/PhysRefData/Handbook/Tables/potassiumtable1.htm
Wavenumbers from http://physics.nist.gov/PhysRefData/Handbook/Tables/potassiumtable5.htm
hfs data from E. Arimondo, M. Inguscio, and P. Violino, Rev. Mod. Phys. 49, 31--75 (1977) http://link.aps.org/doi/10.1103/RevModPhys.49.31
lifetimes from U. Volz and H. Schmoranzer, Phys. Scr. T65, 48 (1996) http://dx.doi.org/10.1088/0031-8949/1996/T65/007
*)

state = {"K", 39, {"Ar", {4, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = .932581;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 2\[Pi] 230.8598601 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"K", 39, {"Ar", {4, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = .932581;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (26.79 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 12985.186 1/Centimeter;
HyperfineA[state] = 2\[Pi] 28.85 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"K", 39, {"Ar", {4, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = .932581;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (26.45 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 13042.896 1/Centimeter;
HyperfineA[state] = 2\[Pi] 6.06 Mega 1/Second;
HyperfineB[state] = 2\[Pi] 2.83 Mega 1/Second;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

(* Rb *)

state = {"Rb", 87, {"Kr", {5, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = 0.27835;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 2\[Pi] 3417.341305452145 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"Rb", 87, {"Kr", {5, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = 0.27835;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (27.679 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 12578.95098147 1/Centimeter;
HyperfineA[state] = 2\[Pi] 407.24 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"Rb", 87, {"Kr", {5, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = 0.27835;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (26.2348 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 12816.54938993 1/Centimeter;
HyperfineA[state] = 2\[Pi] 84.7185 Mega 1/Second;
HyperfineB[state] = 2\[Pi] 12.4965 Mega 1/Second;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"Rb", 85, {"Kr", {5, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = 0.72165;
NuclearSpin[state] = 5/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 2\[Pi] 1011.9108130 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"Rb", 85, {"Kr", {5, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = 0.72165;
NuclearSpin[state] = 5/2;
NaturalWidth[state] = (27.679 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 12578.9483900 1/Centimeter;
HyperfineA[state] = 2\[Pi] 120.527 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"Rb", 85, {"Kr", {5, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = 0.72165;
NuclearSpin[state] = 5/2;
NaturalWidth[state] = (26.2348 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 12816.54678496 1/Centimeter;
HyperfineA[state] = 2\[Pi] 25.0020 Mega 1/Second;
HyperfineB[state] = 2\[Pi] 25.790 Mega 1/Second;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

(* Cs *)

state = {"Cs", 133, {"Xe", {6, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = 1;
NuclearSpin[state] = 7/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 2\[Pi] 2298.1579425 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Even;

state = {"Cs", 133, {"Xe", {6, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = 1.;
NuclearSpin[state] = 7/2;
NaturalWidth[state] = (34.791 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 11178.2681607 1/Centimeter;
HyperfineA[state] = 2\[Pi] 291.9201 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

state = {"Cs", 133, {"Xe", {6, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = 1.;
NuclearSpin[state] = 7/2;
NaturalWidth[state] = (30.405 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 11732.3071049 1/Centimeter;
HyperfineA[state] = 2\[Pi] 50.28827 Mega 1/Second;
HyperfineB[state] = 2\[Pi] (-0.4934) Mega 1/Second;
HyperfineC[state] = 2\[Pi] 0.000560 Mega 1/Second;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];
Parity[state] = Odd;

(* Cu 63 *)
(* HFS data from:
	Spectrochimica Acta Part B: Atomic Spectroscopy Volume 48, Issue 10, August 1993, Pages 1259-1264  
	Phys. Rev. 105, 581-588 (1957), 
	Optics Communications Volume 31, Issue 1, October 1979, Pages 28-30   
   Wavenumber data from NIST website
   Lifetime data from L J Curtis et al 1976 Phys. Scr. 13 109,
   Abundance data from http://www.sisweb.com/referenc/source/exactmas.htm
   *)

state = {"Cu", 63, {"Ar", {3, "d", 10}, {4, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = 0.6917;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 2\[Pi] 5866.92 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

state = {"Cu", 63, {"Ar", {3, "d", 10}, {4, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = 0.6917;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (7.6 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 30535.3 1/Centimeter;
HyperfineA[state] = 2\[Pi] 505.6 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

state = {"Cu", 63, {"Ar", {3, "d", 10}, {4, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = 0.6917;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (7.6 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 30783.7 1/Centimeter;
HyperfineA[state] = 2\[Pi] 194.9 Mega 1/Second;
HyperfineB[state] = 2\[Pi] (-29.1) Mega 1/Second;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

state = {"Cu", 63, {"Ar", {3, "d", 9}, {4, "s", 2}}, {2, "D", 3/2}};
GetStateData[state];
Abundance[state] = 0.6917;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = Null;
Wavenumber[state] = 13245.4 1/Centimeter;
HyperfineA[state] = 2\[Pi] Convert[SpeedOfLight 0.0614 1/Centimeter, Mega 1/Second];
HyperfineB[state] = 2\[Pi] Convert[SpeedOfLight 1.88 10^(-4) 1/Centimeter, Mega 1/Second];
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

state = {"Cu", 63, {"Ar", {3, "d", 9}, {4, "s", 2}}, {2, "D", 5/2}};
GetStateData[state];
Abundance[state] = 0.6917;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = Null;
Wavenumber[state] = 11202.6 1/Centimeter;
HyperfineA[state] = 2\[Pi] Convert[SpeedOfLight 0.0249 1/Centimeter, Mega 1/Second];
HyperfineB[state] = 2\[Pi] Convert[SpeedOfLight 9 10^(-5) 1/Centimeter, Mega 1/Second];
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

(* Cu 65*)

state = {"Cu", 65, {"Ar", {3, "d", 10}, {4, "s"}}, {2, "S", 1/2}};
GetStateData[state];
Abundance[state] = 0.3083;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = 0;
Wavenumber[state] = 0;
HyperfineA[state] = 2\[Pi] 6284.41 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

state = {"Cu", 65, {"Ar", {3, "d", 10}, {4, "p"}}, {2, "P", 1/2}};
GetStateData[state];
Abundance[state] = 0.3083;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (7.6 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 30535.3 1/Centimeter;
HyperfineA[state] = 2\[Pi] 542.9 Mega 1/Second;
HyperfineB[state] = 0;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

state = {"Cu", 65, {"Ar", {3, "d", 10}, {4, "p"}}, {2, "P", 3/2}};
GetStateData[state];
Abundance[state] = 0.3083;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = (7.6 10^-3)^-1 Mega 1/Second;
Wavenumber[state] = 30783.7 1/Centimeter;
HyperfineA[state] = 2\[Pi] 207.9 Mega 1/Second;
HyperfineB[state] = 2\[Pi] (-26.9) Mega 1/Second;
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

state = {"Cu", 65, {"Ar", {3, "d", 9}, {4, "s", 2}}, {2, "D", 3/2}};
GetStateData[state];
Abundance[state] = 0.3083;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = Null;
Wavenumber[state] = 13245.4 1/Centimeter;
HyperfineA[state] = 2\[Pi] Convert[SpeedOfLight 0.0656 1/Centimeter, Mega 1/Second];
HyperfineB[state] = 2\[Pi] Convert[SpeedOfLight 9 10^(-5) 1/Centimeter, Mega 1/Second];
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

state = {"Cu", 65, {"Ar", {3, "d", 9}, {4, "s", 2}}, {2, "D", 5/2}};
GetStateData[state];
Abundance[state] = 0.3083;
NuclearSpin[state] = 3/2;
NaturalWidth[state] = Null;
Wavenumber[state] = 11202.6 1/Centimeter;
HyperfineA[state] = 2\[Pi] Convert[SpeedOfLight 0.0252 1/Centimeter, Mega 1/Second];
HyperfineB[state] = 2\[Pi] Convert[SpeedOfLight 3.7 10^(-5) 1/Centimeter, Mega 1/Second];
Energy[state] = 2\[Pi] Convert[SpeedOfLight Wavenumber[state], Mega 1/Second];

AtomicTransition[atom_String, line_String] := {gstate, estate} = Switch[{atom, line},
    {"H", "D1"},
    {{"H", 1, {{1, "s"}}, {2, "S", 1/2}}, {"H", 1, {{2, "p"}}, {2, "P", 1/2}}},
    {"H", "D2"},
    {{"H", 1, {{1, "s"}}, {2, "S", 1/2}}, {"H", 1, {{2, "p"}}, {2, "P", 3/2}}},
    {"Na"|"Na23", "D1"},
    {{"Na", 23, {"Ne", {3, "s"}}, {2, "S", 1/2}}, {"Na", 23, {"Ne", {3, "p"}}, {2, "P", 1/2}}},
    {"Na"|"Na23", "D2"},
    {{"Na", 23, {"Ne", {3, "s"}}, {2, "S", 1/2}}, {"Na", 23, {"Ne", {3, "p"}}, {2, "P", 3/2}}},
    {"K"|"K39", "D1"},
    {{"K", 39, {"Ar", {4, "s"}}, {2, "S", 1/2}}, {"K", 39, {"Ar", {4, "p"}}, {2, "P", 1/2}}},
    {"K"|"K39", "D2"},
    {{"K", 39, {"Ar", {4, "s"}}, {2, "S", 1/2}}, {"K", 39, {"Ar", {4, "p"}}, {2, "P", 3/2}}},
    {"Rb87", "D1"},
    {{"Rb", 87, {"Kr", {5, "s"}}, {2, "S", 1/2}}, {"Rb", 87, {"Kr", {5, "p"}}, {2, "P", 1/2}}},
    {"Rb87", "D2"},
    {{"Rb", 87, {"Kr", {5, "s"}}, {2, "S", 1/2}}, {"Rb", 87, {"Kr", {5, "p"}}, {2, "P", 3/2}}},
    {"Rb85", "D1"},
    {{"Rb", 85, {"Kr", {5, "s"}}, {2, "S", 1/2}}, {"Rb", 85, {"Kr", {5, "p"}}, {2, "P", 1/2}}},
    {"Rb85", "D2"},
    {{"Rb", 85, {"Kr", {5, "s"}}, {2, "S", 1/2}}, {"Rb", 85, {"Kr", {5, "p"}}, {2, "P", 3/2}}},
    {"Cs"|"Cs133", "D1"},
    {{"Cs", 133, {"Xe", {6, "s"}}, {2, "S", 1/2}}, {"Cs", 133, {"Xe", {6, "p"}}, {2, "P", 1/2}}},
    {"Cs"|"Cs133", "D2"},
    {{"Cs", 133, {"Xe", {6, "s"}}, {2, "S", 1/2}}, {"Cs", 133, {"Xe", {6, "p"}}, {2, "P", 3/2}}},
    _,
    Print["Invalid atom or transition."];Abort[];
];


End[];

EndPackage[];
