(**************************************************************************

   	This file is part of the AtomicDensityMatrix package for Mathematica
    Copyright (C) 2004-2014 Simon M. Rochester <simon@rochesterscientific.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************)


BeginPackage["AtomicDensityMatrix`OpticalField`", {
    "AtomicDensityMatrix`",
    "AtomicDensityMatrix`Common`",
    "AtomicDensityMatrix`TensorAlgebra`"}];

Begin["`Private`"];

(*Optical field*)

Options[OpticalField] = {PolarizationVector -> {1, 0, 0},
    PropagationVector -> {0, 0, 1},
    Parameterization -> "AngleEllipticity",
    CartesianCoordinates -> {Xx, Yy, Zz}, TimeVariable -> Automatic};

(*Takes the arguments and options to OpticalField and puts them in standardized form.*)

optValue[name_, opts_, defopts_, masteropts_] := name /. opts /. defopts /. Automatic -> name /. masteropts;

standardizeField[args_, opts_, defopts_] /; 2 <= Length[args] <= 3 :=
    Module[{
        args1 = EnsureList /@ args,
        parameterization = optValue[Parameterization, opts, defopts, Options[OpticalField]],
        defpol
    },
        defpol = Switch[
            parameterization,
            "AngleEllipticity", {0, 0},
            "RatioPhase", {0, 0}, "Stokes", {1, 0, 0},
            "SphericalPhase", {0, 0}
        ];
        {
            Join[args1[[1]], Drop[{0, 0}, Length[args1[[1]]]]],
            Join[args1[[2]], Drop[{0, 0}, Length[args1[[2]]]]],
            If[Length[args1] == 3, Join[args1[[3]], Drop[defpol, Length[args1[[3]]]]], defpol],
            Simplify[# / Sqrt[#.#]]& /@ (optValue[#, opts, defopts, Options[OpticalField]]& /@ {PolarizationVector, PropagationVector}),
            parameterization
        }
    ];


(* Use explicit Rodgrigues rotation matrix rather than RotationMatrix in OpticalField because the latter 
   assumes complex numbers and so produces very complicated expressions. *)
RodriguesRotationMatrix[th_, v_] :=
    Module[{ident = IdentityMatrix[3], omegaMat},
        omegaMat = Cross[#, v] & /@ ident;
        ident + omegaMat Sin[th] + omegaMat.omegaMat (1 - Cos[th])
    ];

OpticalField[args__, opts : OptionsPattern[]] /; 2 <= Length[{args}] <= 3 :=
    Module[{w, k0, E0, e, b, k, r, al, ep, chi, cphi, ph, S1, S2, S3, f, pol, t, Ep, Em, php, phm, v0, parameterization},
        {{w, k0}, {E0, cphi}, pol, {e, k}, parameterization} = standardizeField[{args}, {opts}, Options[OpticalField]];
        If[!PossibleZeroQ[e.k], Message[OpticalField::trans]];
        t = OptionValue[TimeVariable] /. Automatic -> TimeVariable /. Options[DensityMatrix];
        r = OptionValue[CartesianCoordinates];
        b = Cross[k, e];
        f = E0 E^(I (k0 k.r + cphi - t w));
        Switch[parameterization,
            "AngleEllipticity",
            {al, ep} = pol;
            f RodriguesRotationMatrix[al, k].(Cos[ep] e + I Sin[ep] b),
            "RatioPhase",
            {chi, ph} = pol;
            f (Cos[chi] e + Sin[chi] E^(I ph) b),
            "Stokes",
            {S1, S2, S3} = pol;
            f (b E^(I If[S2 === S3 === 0, \[Pi] / 4, ArcTan[S2, S3]]) Sqrt[1 / 2 - S1 / 2] + (e Sqrt[1 + S1]) / Sqrt[2]),
            "SphericalPhase",
            {Ep, php} = {E0, cphi}; {Em, phm} = pol;
            v0 = ToCartesian[{{E^(I php) Ep}, {0}, {E^(I phm) Em}}];
            E^(I (k0 k.r - t w)) (v0[[1]] e + v0[[2]] b)
        ]
    ];

ReparameterizeField[{S1_, S2_, S3_}, "Stokes" -> "RatioPhase"] :=
    {ArcCos[Sqrt[1 + S1] / Sqrt[2]], If[N@S2 === N@S3 === 0., \[Pi] / 4, ArcTan[S2, S3]]};

ReparameterizeField[{chi_, ph_}, "RatioPhase" -> "Stokes"] :=
    {Cos[2 chi], Cos[ph] Sin[2chi], Sin[ph] Sin[2 chi]};

ReparameterizeField[{al_, ep_}, "AngleEllipticity" -> "Stokes"] :=
    {Cos[2 al] Cos[2 ep], Cos[2 ep] Sin[2 al], Sin[2 ep]};

(*ReparameterizeField[{S1_,S2_,S3_},"Stokes"->"AngleEllipticity"] := 
	{
		If[N@S1===N@S2===0.,0,(Sign[S2]+KroneckerDelta[S2])ArcSin[Sqrt[1/2-S1/(2 Sqrt[S1^2+S2^2+.1])]]],
		(Sign[S3]+KroneckerDelta[S3])ArcCos[Sqrt[1/2+1/2 Sqrt[S1^2+S2^2]]]
	}*)

ReparameterizeField[{S1_, S2_, S3_}, "Stokes" -> "AngleEllipticity"] :=
    {
        If[N@S1 === N@S2 === 0., 0, ArcTan[S1, S2]] / 2,
        ArcSin[S3] / 2
    };

(*ReparameterizeField[{chi_,ph_},"RatioPhase"->"AngleEllipticity"] := 
	ReparameterizeField[ReparameterizeField[{chi,ph},"RatioPhase"->"Stokes"],"Stokes"->"AngleEllipticity"]

ReparameterizeField[{al_,ep_},"AngleEllipticity"->"RatioPhase"] := 
	ReparameterizeField[ReparameterizeField[{al,ep},"AngleEllipticity"->"Stokes"],"Stokes"-> "RatioPhase"]
*)

ReparameterizeField[{chi_, ph_}, "RatioPhase" -> "AngleEllipticity"] :=
    {ArcTan[Tan[2 chi] Cos[ph]] / 2, ArcSin[Sin[2 chi] Sin[ph]] / 2};

ReparameterizeField[{al_, ep_}, "AngleEllipticity" -> "RatioPhase"] :=
    {ArcCos[Cos[2 al] Cos[2 ep]] / 2, ArcTan[Csc[2 al] Tan[2 ep]]};

End[];

EndPackage[];
